From 6eacdafcb03310b4a27c0d5eed3a570fc9e65e2e Mon Sep 17 00:00:00 2001
From: Jean-Paul Saman <jean-paul.saman@m2x.nl>
Date: Fri, 9 Mar 2012 10:20:52 +0100
Subject: [PATCH 6/7] modules/control/dbus: add EsStatus property to MPRIS
 interface.

Report status of main ES-es for current input. A test case has
been added to mpris.py.
---
 extras/misc/mpris.py                   |   10 +++
 extras/misc/mpris.xml                  |   17 ++++++
 modules/control/dbus/dbus.c            |    1 +
 modules/control/dbus/dbus_introspect.h |    1 +
 modules/control/dbus/dbus_player.c     |   99 ++++++++++++++++++++++++++++++++
 modules/control/dbus/dbus_player.h     |    5 ++
 6 files changed, 133 insertions(+), 0 deletions(-)

diff --git a/extras/misc/mpris.py b/extras/misc/mpris.py
index d7063e2..c77a9be 100755
--- a/extras/misc/mpris.py
+++ b/extras/misc/mpris.py
@@ -96,6 +96,13 @@ def TrackChange(Track):
     l_artist.set_text(a)
     l_title.set_text(t)
 
+def TrackEsChange(Track):
+    try:
+        s = Track('audio-es')
+    except:
+        s = 'unknown'
+    l_audio.set_text(s)
+
 # Connects to the Media Player we detected
 def Connect(name):
     global root, player, tracklist, props
@@ -115,9 +122,11 @@ def Connect(name):
     if PropGet('PlaybackStatus') == 'Playing':
         playing = True
         TrackChange(PropGet('Metadata'))
+        TrackEsChange(PropGet('EsStatus'))
 
     window.set_title(props.Get(mpris, 'Identity'))
 
+
 #plays an element
 def AddTrack(widget):
     mrl = e_mrl.get_text()
@@ -261,6 +270,7 @@ bt_stop     = xml.get_object('stop')
 bt_toggle   = xml.get_object('toggle')
 bt_mrl      = xml.get_object('AddMRL')
 bt_shuffle  = xml.get_object('shuffle')
+l_audio     = xml.get_object('l_audio')
 l_artist    = xml.get_object('l_artist')
 l_title     = xml.get_object('l_title')
 e_mrl       = xml.get_object('mrl')
diff --git a/extras/misc/mpris.xml b/extras/misc/mpris.xml
index 6e732b4..9f8566a 100644
--- a/extras/misc/mpris.xml
+++ b/extras/misc/mpris.xml
@@ -406,6 +406,23 @@
                       </packing>
                     </child>
                     <child>
+                      <object class="GtkLabel" id="l_audio">
+                        <property name="width_request">180</property>
+                        <property name="height_request">20</property>
+                        <property name="visible">True</property>
+                        <property name="can_focus">False</property>
+                        <property name="justify">center</property>
+                        <property name="wrap">True</property>
+                        <property name="ellipsize">middle</property>
+                        <property name="single_line_mode">True</property>
+                      </object>
+                      <packing>
+                        <property name="expand">True</property>
+                        <property name="fill">True</property>
+                        <property name="position">3</property>
+                      </packing>
+                    </child>
+                    <child>
                       <object class="GtkEventBox" id="eventicon">
                         <property name="visible">True</property>
                         <property name="can_focus">False</property>
diff --git a/modules/control/dbus/dbus.c b/modules/control/dbus/dbus.c
index b6eac3d..b020ccc 100644
--- a/modules/control/dbus/dbus.c
+++ b/modules/control/dbus/dbus.c
@@ -537,6 +537,7 @@ static void ProcessEvents( intf_thread_t *p_intf,
             break;
         case SIGNAL_STATE:
             vlc_dictionary_insert( &player_properties, "PlaybackStatus", NULL );
+            vlc_dictionary_insert( &player_properties, "EsStatus", NULL );
             break;
         case SIGNAL_RATE:
             vlc_dictionary_insert( &player_properties, "Rate", NULL );
diff --git a/modules/control/dbus/dbus_introspect.h b/modules/control/dbus/dbus_introspect.h
index 31f4c9b..aef6948 100644
--- a/modules/control/dbus/dbus_introspect.h
+++ b/modules/control/dbus/dbus_introspect.h
@@ -72,6 +72,7 @@ static const char* psz_introspection_xml =
 "  <interface name=\"org.mpris.MediaPlayer2.Player\">\n"
 "    <property name=\"Metadata\" type=\"a{sv}\" access=\"read\" />\n"
 "    <property name=\"PlaybackStatus\" type=\"s\" access=\"read\" />\n"
+"    <property name=\"EsStatus\" type=\"a{sv}\" access=\"read\" />\n"
 "    <property name=\"LoopStatus\" type=\"s\" access=\"readwrite\" />\n"
 "    <property name=\"Volume\" type=\"d\" access=\"readwrite\" />\n"
 "    <property name=\"Shuffle\" type=\"d\" access=\"readwrite\" />\n"
diff --git a/modules/control/dbus/dbus_player.c b/modules/control/dbus/dbus_player.c
index 4e4d4b9..3be9bee 100644
--- a/modules/control/dbus/dbus_player.c
+++ b/modules/control/dbus/dbus_player.c
@@ -29,6 +29,10 @@
 #endif
 
 #include <vlc_common.h>
+#include <vlc_es.h>
+#include <vlc_arrays.h>
+
+#include <vlc_input.h>
 #include <vlc_playlist.h>
 #include <vlc_interface.h>
 #include <vlc_aout_intf.h>
@@ -710,6 +714,92 @@ DBUS_METHOD( Metadata )
     REPLY_SEND;
 }
 
+#define ADD_ES( entry, type, data ) \
+    if( data ) { \
+        dbus_message_iter_open_container( &dict, DBUS_TYPE_DICT_ENTRY, \
+                NULL, &dict_entry ); \
+        dbus_message_iter_append_basic( &dict_entry, DBUS_TYPE_STRING, \
+                &ppsz_es_cat[entry] ); \
+        dbus_message_iter_open_container( &dict_entry, DBUS_TYPE_VARIANT, \
+                type##_AS_STRING, &variant ); \
+        dbus_message_iter_append_basic( &variant, \
+                type, \
+                & data ); \
+        dbus_message_iter_close_container( &dict_entry, &variant ); \
+        dbus_message_iter_close_container( &dict, &dict_entry ); }
+
+static void
+MarshalEsStatus( intf_thread_t *p_intf, DBusMessageIter *args )
+{
+    DBusMessageIter dict, dict_entry, variant;
+
+    const char *psz_es_status;
+    const char* ppsz_es_cat[] = { "unknown-es", "video-es", "audio-es", "spu-es", "nav-es" };
+    const int cat[] = { UNKNOWN_ES, VIDEO_ES, AUDIO_ES, SPU_ES, NAV_ES };
+
+    dbus_message_iter_open_container( args, DBUS_TYPE_ARRAY, "{sv}", &dict );
+
+    input_thread_t *p_input = playlist_CurrentInput( p_intf->p_sys->p_playlist );
+    if( p_input )
+    {
+        for( unsigned int i = 0; i < ARRAY_SIZE(cat); i++ )
+        {
+            input_es_state_e es_state = input_GetEsState( p_input, cat[i] );
+            switch( es_state )
+            {
+                case INPUT_ES_STATE_DISABLED:
+                    psz_es_status = ES_STATUS_DISABLED;
+                    break;
+                case INPUT_ES_STATE_ENABLED:
+                    psz_es_status = ES_STATUS_ENABLED;
+                    break;
+                case INPUT_ES_STATE_ERROR:
+                    psz_es_status = ES_STATUS_ERROR;
+                    break;
+                default:
+                    psz_es_status = ES_STATUS_ERROR;
+                    break;
+            }
+            ADD_ES( i, DBUS_TYPE_STRING, psz_es_status );
+        }
+        vlc_object_release( (vlc_object_t*) p_input );
+    }
+    else
+    {
+        psz_es_status = ES_STATUS_UNKNOWN;
+        for( unsigned int i = 0; i < ARRAY_SIZE(cat); i++ )
+        {
+            ADD_ES( i, DBUS_TYPE_STRING, psz_es_status );
+        }
+    }
+
+    dbus_message_iter_close_container( args, &dict );
+}
+#undef ADD_ES
+
+DBUS_METHOD( EsStatus )
+{
+    REPLY_INIT;
+    OUT_ARGUMENTS;
+
+    DBusMessageIter v/*, a*/;
+
+    if( !dbus_message_iter_open_container( &args, DBUS_TYPE_VARIANT,
+                                           "a{sv}", &v ) )
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+
+    MarshalEsStatus( p_this, &v );
+
+    if( /*( !dbus_message_iter_open_container( &v, DBUS_TYPE_ARRAY, "{sv}", &a ) ||
+          !dbus_message_iter_close_container( &v, &a ) ) ||*/
+
+        !dbus_message_iter_close_container( &args, &v ) ) {
+        return DBUS_HANDLER_RESULT_NEED_MEMORY;
+    }
+
+    REPLY_SEND;
+}
+
 /******************************************************************************
  * Seeked: non-linear playback signal
  *****************************************************************************/
@@ -770,6 +860,7 @@ DBUS_METHOD( GetProperty )
     PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "Metadata", Metadata )
     PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "Position", Position )
     PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "PlaybackStatus", PlaybackStatus )
+    PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "EsStatus", EsStatus )
     PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "LoopStatus", LoopStatusGet )
     PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "Shuffle", ShuffleGet )
     PROPERTY_FUNC( DBUS_MPRIS_PLAYER_INTERFACE, "Volume", VolumeGet )
@@ -910,6 +1001,14 @@ PropertiesChangedSignal( intf_thread_t    *p_intf,
 
             dbus_message_iter_close_container( &entry, &variant );
         }
+        else if( !strcmp( ppsz_properties[i], "EsStatus" ) )
+        {
+            dbus_message_iter_open_container( &entry,
+                                              DBUS_TYPE_VARIANT, "a{sv}",
+                                              &variant );
+            MarshalEsStatus( p_intf, &variant );
+            dbus_message_iter_close_container( &entry, &variant );
+        }
         else if( !strcmp( ppsz_properties[i], "PlaybackStatus" ) )
         {
             dbus_message_iter_open_container( &entry,
diff --git a/modules/control/dbus/dbus_player.h b/modules/control/dbus/dbus_player.h
index f67ae62..0744f68 100644
--- a/modules/control/dbus/dbus_player.h
+++ b/modules/control/dbus/dbus_player.h
@@ -41,6 +41,11 @@
 #define PLAYBACK_STATUS_PAUSED  "Paused"
 #define PLAYBACK_STATUS_ERROR   "Error"
 
+#define ES_STATUS_UNKNOWN  "Unknown"
+#define ES_STATUS_DISABLED "Disabled"
+#define ES_STATUS_ENABLED  "Enabled"
+#define ES_STATUS_ERROR    "Error"
+
 /* Handle incoming dbus messages */
 DBusHandlerResult handle_player ( DBusConnection *p_conn,
                                   DBusMessage *p_from,
-- 
1.7.7.6

